/*!
 *  raweGauge v1.5
 *  http://www.redigital.co.uk
 *  Date: August 2014
 *
 *  Copyright (c) 2014 Richard A W Edwards
 *  Licensed under MIT
 *  http://opensource.org/licenses/MIT
 *
 *  @REdigitalUK
 */

(function($, document){

    var Gauge = {
        /* !Initialise plugin */
        init : function(el){
            var base = this;
            base.defineVars(el);
            base.build();
        },

        /* !Define initial variables */
        defineVars : function(el){
            var base = this;
            base.el = el;
            base.$el = $(el);
            base.$inputs = base.$el.find('input');
            base.$labels = base.$el.find('label');
            base.haschecked = false;
            base.cssTransitions = base.detectCssTransitions();
        },
        
        /* !Detect CSS transitions */
        detectCssTransitions : function () {
			var b = document.body || document.documentElement,
				s = b.style,
				p = 'transition';		
			if (typeof s[p] === 'string') { 
				return true; 
			}
			return false;
		},

        /* !Build the HTML */
        build : function(){
            var base = this,
                container = document.createElement('div'),
                checkedNumber = "";

            // Hide radios and labels
            base.$inputs.add(base.$labels).hide();

            // Create container & loupe
            container.className = 'gaugeItems';
            base.$el.append(container);
            base.$items = base.$el.find('.gaugeItems');
            base.$items.append('<div class="gaugeLoupe"></div>');
            base.$loupe = base.$el.find('.gaugeLoupe');

            // Create an item for each radio button
            base.$inputs.each(function(i){
                var ic = i + 1, //index corrected
                    selected = "",
                    value = $(this).prop('value');
                if ($(this).prop('checked') === true){
                    selected = 'selected';
                    base.haschecked = true;
                    checkedNumber = ic;
                }
                base.$items.append('<div class="item item' + value + ' ' + (selected ? selected : '') + ' ">' + value + '</div>');
            });

            // Get item widths and item border widths to use later
            base.$itemsChildren = base.$items.children('.item');
            base.itemWidth = base.$itemsChildren.css('width').replace(/\D/g,'');
            base.itemBorder = base.$itemsChildren.css('border-width').replace(/\D/g,'');

            // If a radio is checked, set loupe over relative item - using no animation
            base.haschecked ? base.goTo(checkedNumber, false) : base.$loupe.hide();

            base.defineEvents();
        },

        /* !Events */
        defineEvents : function(){
            var base = this;
            base.$itemsChildren.on('click', {base: base} , base.handler);
        },

        /* !Toggle CSS animation on/off */
        toggleAnim : function(){
            var base = this,
                prop = base.$loupe.css('transition-property');
            if(prop === 'none'){
                base.$loupe.css('transition-property', 'all');
            } else {
                base.$loupe.css('transition-property', 'none');
            }
        },

        /* !Go To */
        // Sets loupe over selected item
        // param: number - the index of the item to select (number)
        // param: anim - true to animate, false to set immediately (bool)
        goTo : function(number, anim){
            var base = this,
	            number = number - 1,
	            leftPosition = (base.itemWidth * number) + (number * base.itemBorder);

			if (anim === false){
	            base.toggleAnim();
	            base.$loupe.css('left', leftPosition );
	            base.toggleAnim();
            } else {
            	if(base.cssTransitions === false){	            	
	            	//no CSS transition - older browsers
	            	base.$loupe.animate({ left: leftPosition }, 500); 
            	} else {
            		base.$loupe.css('left', leftPosition );
	            }	            
            }
            // show loupe after animation (on first selection)
            setTimeout(function(){
            	base.$loupe.css('display', 'block');
            },500);
        },

        /* !Click Handler */
        handler : function(event){
            var base = event.data.base,
                $this = $(this),
                itemindex = $this.index(),
                itemnumber = itemindex - 1;
                
            base.$inputs.eq(itemnumber).prop('checked',true); // check relative radio button
            base.$itemsChildren.removeClass('selected');
            base.goTo(itemindex);

            // Add 'selected' class to selected item (for styling)
            var delay = setTimeout(setclass, 500);
            function setclass(){
                $this.addClass('selected');
            }
            event.preventDefault();
        }
    };

    if (typeof Object.create !== "function") {
        Object.create = function (obj) {
            function F() {}
            F.prototype = obj;
            return new F();
        };
    }

    $.fn.raweGauge = function(){
        return this.each(function () {
            // Check raweGauge not already invoked on item
            if ($(this).data("raweGaugeInit") === true) {
                return false;
            }
            $(this).data("raweGaugeInit", true);
            var gauge = Object.create(Gauge);
            gauge.init(this);
            $.data(this, "raweGauge", gauge);
        });
    };

}(jQuery, document));